/*************************************************************************************************************************************************/
/*!      TIDA-00648
 *
 *       4-20mA Current Loop Transmitter Reference Design
 *
 *
 */
/**************************************************************************************************************************************************
 *       Copyright  2015 Texas Instruments Incorporated - http://www.ti.com/                                                                      *
 ***************************************************************************************************************************************************
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: *
 *                                                                                                                                                 *
 *    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.                 *
 *                                                                                                                                                 *
 *    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the        *
 *    documentation and/or other materials provided with the distribution.                                                                         *
 *                                                                                                                                                 *
 *    Neither the name of Texas Instruments Incorporated nor the names of its contributors may be used to endorse or promote products derived      *
 *    from this software without specific prior written permission.                                                                                *
 *                                                                                                                                                 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT          *
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     *
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT         *
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY    *
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE      *
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                                                           *
 **************************************************************************************************************************************************/
//
//   TIDA-00648 v1.0
//
//   Texas Instruments Inc.
//   August 2015
//   Built with Code Composer Studio V6.1
//*************************************************************************************************************************************************


#include <msp430.h>
#include "DAC161.h"
#include "hal.h"


unsigned char dacStatus = 0;
unsigned char DAC_Error_Processing;
unsigned long dacValue = 0;
unsigned long dacValue_mA = 3;
volatile unsigned char buttonDebounce;
//const float DacGainCorrection = 1.0;
//const float DacOffsetCorrection = 0.0;

void Ports_init(void);
void USCIA0_init(void);


void main(void)
{
	WDTCTL = WDTPW | WDTHOLD;                // Stop watchdog timer

	Ports_init();
	USCIA0_init();

	__enable_interrupt();

	// Configure the DAC161S997
	Setup_DAC161  (DAC161_STD_ERR_CONFIG_MASKED, 3370, 21800);
	DAC_Error_Processing = 0;

	DAC161_Set_Out_Value (0);

	while(1)
	{
		// In case we extend the time between samples in the future, we will still need to keep the DAC
		// from timing out by sending a NOP command every time the loop is processed.
		DAC161_Nop();
		// Did the DAC161 indicate an error?
		if (DAC_Error_Processing)
		{
			// Perform DAC ERROR Processing Here
			DAC_Error_Processing = 0;
			dacStatus = DAC161_Read_Status();         // Read Status to clear error
		}
	}
}


/*************************************************************************************************************************************************
 *  Ports_init
 **************************************************************************************************************************************************/
/*!
 * @brief Initialize the GPIO pins on the MSP430F5172
 *
 * This function is called once to configure the GPIO pins on the system.
 *
 * @return  none
 **************************************************************************************************************************************************/
void Ports_init(void)
{
	//unused pins output low
	P3DIR |= 0xFF;
	PJDIR |= 0xFF;
	P2OUT &= ~ (BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6);
	P2DIR |= (BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6);
	P1OUT &= ~BIT7;
	P1DIR |= BIT7;
	PJOUT = 0x0000;
	PJDIR = 0xFFFF;
	P1SEL |= BIT1+BIT2;             // P1.1,2 = USCI_A0 TXD/RXD

	DAC_CS_PORTOUT |= DAC_CS_N;     // Active Low, so set high
	DAC_CS_PORTDIR |= DAC_CS_N;
	USCIB0_setupMasterSPI();

	P1DIR &= ~(DAC_ERROR_N );       // Input Pins
	P1IES |=  (DAC_ERROR_N );       // High to Low Edge
	P1IFG &= ~0xFF;                 // Clear any pending interrupts
	P1IE  |=  (DAC_ERROR_N );       // Enable Interrupts
	P2DIR &= ~(USER_INTERRUPT_N);   // Input Pins
	P2OUT |=  (USER_INTERRUPT_N);   // internal Pullup
	P2REN |=  (USER_INTERRUPT_N);   // enable internal Pullup
	P2IES |=  (USER_INTERRUPT_N);   // High to Low Edge
	P2IFG &= ~0xFF;                 // Clear any pending interrupts
	P2IE  |=  (USER_INTERRUPT_N);   // Enable Interrupts

	buttonDebounce = 1;
}

/*************************************************************************************************************************************************
 *  USCIA0_init
 **************************************************************************************************************************************************/
/*!
 * @brief Initialize the USCI_A0 module in UART Mode
 *
 * This function is called once to configure the USCI_A0 Module.
 *
 * @return  none
 **************************************************************************************************************************************************/
void USCIA0_init(void)
{
	UCA0CTL1 |= UCSWRST;                      // **Put state machine in reset**
	UCA0CTL1 |= UCSSEL_2;                     // SMCLK 1MHz
	UCA0BR0 = 9;                              // 115200 baudrate (see User's Guide)
	UCA0BR1 = 0;                              // 115200 baudrate
	UCA0MCTL |= UCBRS_1 + UCBRF_0;            // Modulation UCBRSx=1, UCBRFx=0
	UCA0CTL1 &= ~UCSWRST;                     // **Initialize USCI state machine**
	UCA0IE |= UCRXIE;                         // Enable USCI_A0 RX interrupt
}

/*************************************************************************************************************************************************
 *  startWDT
 **************************************************************************************************************************************************/
/*!
 * @brief start watchdog timer
 *
 * WDT as 250ms interval counter
 *
 * @return  none
 **************************************************************************************************************************************************/

void startWDT()
{
	//WDT as 250ms interval counter
	SFRIFG1 &= ~WDTIFG;
	WDTCTL = WDT_ADLY_250;                           // WDT 250ms, ACLK, interval timer
	SFRIE1 |= WDTIE;                                 // Enable WDT interrupt
}

/**************************************************************************************************************************************************
 * USCI_A0_ISR
 **************************************************************************************************************************************************/
/*!
 * @brief USCI_A0 Interrupt Routine
 *
 * This interrupt will occur when data is received on USCI_A0.
 *
 * @return None
 **************************************************************************************************************************************************/
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
	char data;
	switch(__even_in_range(UCA0IV,4))
	{
	case 0:break;                              // Vector 0 - no interrupt
	case 2:
		data=UCA0RXBUF;                        // Vector 2 - RXIFG
		if(data == 'U')
		{
			dacValue++;
			DAC161_Set_Out_Value(dacValue);
		}
		else if(data == 'R')
		{
			dacValue=0;
			DAC161_Set_Out_Value(dacValue);
		}

		while (!(UCA0IFG&UCTXIFG));             // USCI_A0 TX buffer ready?
		UCA0TXBUF = data;                       // TX -> RXed character
		break;
	case 4:break;                               // Vector 4 - TXIFG
	default: break;
	}
}

/**************************************************************************************************************************************************
 * PORT1_ISR
 **************************************************************************************************************************************************/
/*!
 * @brief Port 1 Interrupt Routine
 *
 * This interrupt will occur when DAC error occurs.
 *
 * @return None
 **************************************************************************************************************************************************/
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)
{
	// Has an error from the DAC occurred?
	if (P1IFG & DAC_ERROR_N)
	{
		DAC_Error_Processing = 1;
	}

	P1IFG = 0;
}

/**************************************************************************************************************************************************
 * PORT2_ISR
 **************************************************************************************************************************************************/
/*!
 * @brief Port 2 Interrupt Routine
 *
 * This interrupt will occur when the system button is pushed.
 * Every time the button is pressed, the output of the DAC is increased by 1mA, between 4 and 24mA.
 *
 * @return None
 **************************************************************************************************************************************************/
#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR (void)
{
	if (buttonDebounce == 1)
	{
		buttonDebounce = 2;
		startWDT();
		if (P2IFG & USER_INTERRUPT_N)
		{
			// Test code - Ramp from 4mA to 24mA and start over again at 4mA.
			dacValue_mA += 1;
			if (dacValue_mA > 24)
				dacValue_mA = 4;

			dacValue = dacValue_mA * 2730;
			DAC161_Set_Out_Value (dacValue);
		}
	}
	P2IFG = 0;
}


/**************************************************************************************************************************************************
 * WDT_ISR
 **************************************************************************************************************************************************/
/*!
 * @brief WDT Interrupt Service Routine
 *
 * Handles WDT interrupt
 *
 * @return None
 **************************************************************************************************************************************************/

#pragma vector=WDT_VECTOR
__interrupt void WDT_ISR(void)
{
	if (buttonDebounce == 2)
	{
		buttonDebounce = 1;
		SFRIFG1 &= ~WDTIFG;
		SFRIE1 &= ~WDTIE;
		WDTCTL = WDTPW + WDTHOLD;
	}
}
